STYLE = """
<style type="text/css">
  .div-table {
    display: flex;
    flex-direction: column;
  }

  .table {
    width: 100%;
    height: auto;
    margin-top: 10px;
    border-collapse: collapse;
    border: 0.5px solid gray;
  }

  .tr-table {
    text-align: center;
    font-size: 10px;
    padding-left: 10px;
    border: 0.5px solid gray;
  }

  .td-table {
    text-align: center;
    font-size: 10px;
    padding-left: 10px;
    border: 0.5px solid gray;
  }

  .div-header {
    display: flex;
    flex-direction: row;
    justify-content: space-between;
    align-items: center;
  }

  .sub-header-info {
    display: flex;
    flex-direction: column;
    justify-content: end;
    width: 100%;
  }

  .sub-header-text {
    display: flex;
    flex-direction: row;
    align-items: center;
    justify-content: start;
  }

  .sub-header-info-label {
    width: 20%;
    text-align: start;
  }

  .div-sub-header {
    border-top: 1px solid gray;
  }
</style>"""  # noqa